
The next part is to provide a suitable record data structure as well as consistency tests. This is %%@
done within the following module.
\begin{code}
module Goguen where 

import RelAlg
import Lattice
import List (intersect)
import LFuzzyRel
\end{code}
Later on, it will become clear why we make use of \smalltt{Lattice}. The data structure 
\begin{code}
data Gog loos obj mor = Gog { gog_ded   :: Ded obj mor
                             ,gog_up    :: mor -> mor
                             ,gog_down  :: mor -> mor
                             ,gog_derOp :: String -> loos -> mor -> mor -> mor }
\end{code}
directly reflects our type class for Goguen categories. Notice, that {\em standard} definitions for %%@
$^{\ua}$ and $^{\da}$ that are valid for all Goguen categories do not exist. Hence, the user has to %%@
provide these functions. The opposite is true for derived operations. M.\ Winter in \cite{WinterGog1} %%@
gave a standard definition that operates on all scalars of a given object. This is rather %%@
inefficient, and we think that a standard implementation can be omitted. Thus, the user has to %%@
provide this routine himself. Later on, we will give an implementation for the standard model within %%@
the \smalltt{LFuzzyRel} module.

For abbreviated notations, we introduce the following functions.
\begin{code}
gog_isObj    = ded_isObj      .gog_ded
gog_isMor    = ded_isMor      .gog_ded
gog_objects  = ded_objects    .gog_ded
gog_homset   = ded_homset     .gog_ded
gog_source   = ded_source     .gog_ded
gog_target   = ded_target     .gog_ded
gog_idmor    = ded_idmor      .gog_ded
gog_comp     = ded_comp       .gog_ded
gog_converse = ded_converse   .gog_ded
gog_meet     = ded_meet       .gog_ded
gog_incl     = ded_incl       .gog_ded
gog_join     = ded_join       .gog_ded
gog_bottom   = ded_bottom     .gog_ded
gog_rres     = ded_rres       .gog_ded
gog_lres     = ded_lres       .gog_ded
gog_syq      = ded_syq        .gog_ded
gog_top      = ded_top        .gog_ded
gog_divAll   = ded_divAll     .gog_ded
gog_distrAll = divAll_distrAll.gog_divAll
gog_all      = distrAll_all   .gog_distrAll
gog_cat      = all_cat        .gog_all
\end{code}
Thus, the extension of RATH with respect to the new data structures is complete. Our attention now %%@
goes to the consistency tests. Which checks are necessary, can directly be inferred from Definition %%@
\ref{DefnGog}. For Property (1) we only have to check whether $\Top_{AB}\neq\Bot_{AB}$ holds for all %%@
objects $A,B$. The test for Dedekind categories is already provided by RATH. With property (2) the %%@
parts (2a)-(2d) are essential with respect to the $^{\ua}$ and $^{\da}$ operation, because they %%@
guarantee that our interpretation of these two operations is valid. Part (2e) is fundamental for the %%@
$\alpha$-cut Theorem (\ref{ThmACut}) and, thus, plays a key role. But, unfortunately, for any two %%@
objects $A$ and $B$ of the underlying Dedekind category, we need all antimorphisms between %%@
$Sc_{\SG}[A]$ and $Crisp_{\SG}[A,B]$. Obviously, this computation tends to be very inefficient. The %%@
trivial implementation would be to compute all mappings and then test which one is an antimorphism. %%@
Of course, this is unacceptable. Therefore, our first thoughts go to possible improvements.

It is clear that there will be no general function that delivers a certain computation formula for %%@
every antimorphism. Furthermore, an approach to find certain structural interactions between %%@
$Sc_{\SG}[A]$ and $Crisp_{\SG}[A,B]$ will fail. If we, for example, take the standard Goguen category %%@
provided by Theorem \ref{ThmLRelGoguen}, we see that the structure of $Sc_{\SG}[A]$ strongly depends %%@
on the underlying lattice $\SL$. But, the structure of $Crisp_{\SG}[A,B]$ only depends on the objects %%@
$A$ and $B$.

Thus, we have to go another way. Again, our rescue is that we are dealing with finite structures. The %%@
scalars $Sc_{\SG}[A]$ of an arbitrary object $A$ within our Goguen category form a lattice (cf.\ %%@
Lemma \ref{LemScalarLat}). Thus, every scalar can be determined by the corresponding join-irreducible %%@
elements and, hence, any mapping $f:\SL_{1}\lora\SL_{2}$ fulfilling
\[ f(\bigvee M) = \bigwedge_{x\in M}f(x) \mbox{\ for\ } M\sse \SL_{1} \]
can even uniquely be described by its reduction to the join-irreducible elements of $\SL_{1}$. We %%@
therefore only need to compute all (possible) images of $f$ reduced to the join-irreducible elements %%@
of $\SL_{1}$ such that the property above holds. To do so, we have to hold in mind that there can be %%@
an order relationship between two irreducible elements. Let, for example, $x_{1},x_{2}\in\SL_{1}$ be %%@
join-irreducible elements such that $x_{1}\leq_{1}x_{2}$ holds. Then we have to take care that %%@
$f(x_{2})\leq_{2}f(x_{1})$ is true. In the case that $x_{1}$ and $x_{2}$ are incomparable, any %%@
combination of $f(x_{1})$ and $f(x_{2})$ in respect to $\leq$ is possible. Having computed the images %%@
of all join-irreducible elements of $\SL_{1}$, we can compute the image of any element of %%@
$x\in\SL_{1}$ by
\begin{equation}
\label{EquCompAnti}
f(x) = \bigwedge\{\ f(y)\ |\ y\leq_{1} x,\ y\mbox{\ join-irreducible\ }\}.
\end{equation}
Notice the special role of the least element of $\SL_{1}$. We have to guarantee $f(0_{1}) = %%@
f(1_{2})$. This follows immediately if we set $M=\es$ in the equation above.

Hence, we are able to provide a relatively efficient standard function for this purpose. But first, %%@
we need some auxiliary functions. The explanations above indicate that we need our \smalltt{Lattice} %%@
module to compute the set of join-irreducible elements. Therefore, we introduce the following %%@
functions
\begin{code}
gog_poSet g src trg elements = 
                  PoSet { poSet_isElem   = flip elem $ gog_homset g src trg
                         ,poSet_elements = elements
                         ,poSet_lEq      = gog_incl g }
                                      
gog_lat   g src trg elements = let pS = gog_poSet  g src trg elements
                                   bE = gog_bottom g src trg
                                   tE = gog_top    g src trg
                                   m  = gog_meet   g
                                   j  = gog_join   g
                               in Lat   { lat_poSet   = pS
                                         ,lat_sup     = j
                                         ,lat_inf     = m
                                         ,lat_topEl   = gog_top    g src trg
                                         ,lat_botEl   = bE
                                         ,lat_atomS   = atomSetBy  pS bE m []
                                         ,lat_jIrredS = irredSetBy pS bE j []
                                         ,lat_mIrredS = irredSetBy pS tE m [] }
\end{code}
which create the according data structures for the morphisms between \smalltt{src} and \smalltt{trg}. %%@
Furthermore, we need two more functions for the determination of the scalars and crisp relations, %%@
respectively.
\begin{code}
scalars :: (Eq mor) => Ded obj mor -> obj -> [mor]
scalars d a =  filter (\m -> ded_incl d m (ded_idmor d a) && 
                             ded_comp d m (ded_top d a a) == 
                             ded_comp d (ded_top d a a) m) 
                      $ ded_homset d a a

crispRel :: (Eq mor) => Gog l obj mor -> obj -> obj -> [mor]
crispRel g a b = filter (\m -> gog_up g m == m) $ gog_homset g a b
\end{code}
Obviously, they are direct realizations of the underlying definitions. Having this, we can compute %%@
the antimorphisms. Since extensive computations are to expect, we provide a convenient %%@
parametrization such that the user is able to support our functions throughout this module. For %%@
antimorphisms, we provide two routines that are parametrized analogous to our \smalltt{Lattice} %%@
module.
\begin{code}
antiMorphBy :: (Eq mor) => 
               Gog l obj mor -> (Ded   obj mor -> obj -> [mor]       ) ->                               
                                (Gog l obj mor -> obj -> obj -> [mor]) -> 
               obj -> obj -> [mor -> mor]
antiMorphBy g sc crisp src trg = 
  [\x -> foldl1 (gog_meet g) [f_i | (i,f_i)<-m,gog_incl g i x] 
   | m <- anti (sc (gog_ded g) src) (crisp g src trg)]       
  where
    anti os1 os2 = 
      let botG  = gog_bottom g src src
          irred = irredSetBy (gog_poSet g src src os1) 
                             botG (gog_join g) []
      in map ((botG, gog_top g src trg):) $ anti' irred os2 []
    anti' []     os2 _       = [[]]
    anti' (a:os) os2 morph   = [ res:f | f<-anti' os os2 morph, 
                                 res<-foldl1 intersect $ getPart f a os2]
    getPart []           a os2 = [[(a,f_a)| f_a <- os2]]
    getPart ((b,f_b):fs) a os2 
      | gog_incl g a b = [ (a,f_a)| f_a <- os2, gog_incl g f_b f_a] 
                         : getPart fs a os2
      | gog_incl g b a = [ (a,f_a)| f_a <- os2, gog_incl g f_a f_b] 
                         : getPart fs a os2
      | otherwise      = getPart fs a os2 

antiMorph :: (Eq mor) => Gog l obj mor -> obj -> obj -> [mor -> mor]
antiMorph g = antiMorphBy g scalars crispRel
\end{code}
The function \smalltt{antiMorphBy} is, among others, parametrized by two functions to determine the %%@
scalars and crisp relations, respectively. The standard variant then is delivered by %%@
\smalltt{antiMorph}.\\
Obviously, we represent an antimorphism $f$ as a list of tuples $(x,f(x))$. The computation is %%@
divided into three steps. \\
The first one is done by the function \smalltt{getPart}. It is called by \smalltt{anti'} and gets an %%@
(already partially computed) antimorphism \smalltt{f}, which shall be extended by all possible %%@
combinations of an element \smalltt{a} and its image \smalltt{f\_a} such that the new morphisms are %%@
still antimorphisms. To guarantee this, \smalltt{getPart} generates a list of possible tuples %%@
\smalltt{(a,f\_a)} for every single tuple \smalltt{(b,f\_b)}, which is already in the list %%@
representing \smalltt{f}. The intersection of all these lists delivers all possible extensions for %%@
\smalltt{f}. The extension then is done in \smalltt{anti'} where a list of the resulting new %%@
morphisms is generated.\\
The second step is to guarantee that $f(0_{1}) = f(1_{2})$ holds. This is made by the function %%@
\smalltt{anti}, which inserts the pair \smalltt{(gog\_bottom g src src, gog\_top g src trg)} into %%@
every computed list after \smalltt{anti'} is finished.\\
As we only have lists of tuples up to now, we finally have to provide suitable functions that can be %%@
returned. This is done by \smalltt{antiMorphBy} which makes use of Formula \ref{EquCompAnti}. Notice %%@
that the computation in \smalltt{antiMorphBy} definitely succeeds, even for the special case %%@
$f(0_{1})$.

Now, we are ready to test Property (2e) of Definition \ref{DefnGog}. Notice that we (in analogy to %%@
the \smalltt{Lattice} module) prefer to extract this test so that the user has direct access and does %%@
not necessarily need to perform the whole consistency check.
\begin{code}
testAntiMorphBy :: (Eq obj, Eq mor) => Gog l obj mor ->
                   (Gog l obj mor -> obj -> obj -> [mor -> mor]) ->
                   (Ded   obj mor -> obj -> [mor])               ->
                   (Gog l obj mor -> obj -> obj -> [mor])        ->
                   obj -> obj -> TestResult obj mor
testAntiMorphBy g anti sc crisp src trg =
  ffold [let scG   = sc (gog_ded g) src
             inclG = gog_incl g
             supSc = foldl1 (gog_join g) [gog_comp g s (a s) | s<-scG]
         in ffold [test (gog_incl g m supSc == foldl1 (&&) 
                        [gog_incl g (gog_down g (gog_rres g s m)) (a s)|s<-scG]) 
                        [src] [m] "Antimorphisms:"]
         | m <- gog_homset g src trg, 
           a <- anti g src trg]
           
testAntiMorphAllBy :: (Eq obj, Eq mor) => Gog l obj mor -> 
                      (Gog l obj mor -> obj -> obj -> [mor -> mor]) ->
                      (Ded   obj mor -> obj -> [mor])               ->
                      (Gog l obj mor -> obj -> obj -> [mor])        ->
                      TestResult obj mor
testAntiMorphAllBy g anti sc crisp =
  ffold [ testAntiMorphBy g anti sc crisp src trg | src <- gog_objects g,
                                                    trg <- gog_objects g]

testAntiMorph    :: (Eq obj, Eq mor) => 
                    Gog l obj mor -> obj -> obj -> TestResult obj mor
testAntiMorph    g = testAntiMorphBy    g antiMorph scalars crispRel

testAntiMorphAll :: (Eq obj, Eq mor) => Gog l obj mor -> TestResult obj mor
testAntiMorphAll g = testAntiMorphAllBy g antiMorph scalars crispRel
\end{code}
Obviously, we separate the tests twofold. Thus, the user on the one hand has the chance to provide %%@
its own functions for determining the needed antimorphisms, scalars and crisp relations, %%@
respectively. On the other hand, he can decide whether he wants to test every pair of objects of the %%@
underlying Goguen category or if he only wants to check one specific combination of source and %%@
target. The tests themselves are a direct realization of Definition \ref{DefnGog}(2e). Again, the %%@
standard variants are given by \smalltt{testAntiMorph} and \smalltt{testAntiMorphAll}, respectively.

The all-in-one tests for $^{\ua}$ and $^{\da}$ are provided analogously.
\begin{code}
testUpDownBy :: (Eq obj, Eq mor) => Gog l obj mor -> 
                (Gog l obj mor -> obj -> obj -> [mor -> mor]) ->
                (Ded   obj mor -> obj -> [mor])               ->
                (Gog l obj mor -> obj -> obj -> [mor])        ->
                obj -> obj -> obj -> TestResult obj mor
testUpDownBy g anti sc crisp src trg trg2 =
  let incl   = gog_incl     g;   up     = gog_up       g
      down   = gog_down     g;   comp   = gog_comp     g
      source = gog_source   g;   target = gog_target   g
      homset = gog_homset   g;   conv   = gog_converse g
      scG    = sc (gog_ded g) src
  in
   testAntiMorphBy g anti sc crisp src trg .
   ffold [test (s == gog_bottom g src src || up s == gog_idmor g src) 
               [src] [s] "Up(a) = I:" | s<-scG] .
   ffold [ffold [test (source (up   m) == src && target (up   m) == trg) 
                      [src,trg] [m] "Domain/Range Up:" .
                 test (source (down m) == src && target (down m) == trg)
                      [src,trg] [m] "Domain/Range Down:"] .
          ffold [test (incl m (down m2) == incl (up m) m2) 
                      [src,trg] [m,m2] "UpDown-Galois:" 
                 | m2 <- homset src trg]
          |   m <- homset src trg] . 
          ffold [test (up (comp (conv m) (down m2)) == 
                       comp (conv $ up m) (down m2))
                      [src,trg,trg2] [m,m2] "Comp/Conv/UpDown:" 
                 | m  <- homset trg src,
                   m2 <- homset trg trg2]

testUpDownAllBy :: (Eq obj, Eq mor) => Gog l obj mor -> 
                   (Gog l obj mor -> obj -> obj -> [mor -> mor]) ->
                   (Ded   obj mor -> obj -> [mor])               ->
                   (Gog l obj mor -> obj -> obj -> [mor])        ->
                   TestResult obj mor
testUpDownAllBy g anti sc crisp = 
    ffold [ testUpDown g src trg trg2 | src  <- gog_objects g, 
                                        trg  <- gog_objects g,
                                        trg2 <- gog_objects g]

testUpDown :: (Eq obj, Eq mor) => 
              Gog l obj mor -> obj -> obj -> obj -> TestResult obj mor
testUpDown g = testUpDownBy g antiMorph scalars crispRel

testUpDownAll :: (Eq obj, Eq mor) => Gog l obj mor -> TestResult obj mor
testUpDownAll g = testUpDownAllBy g antiMorph scalars crispRel
\end{code}
Hence, we can test Goguen categories as follows.
\begin{code}
gog_TESTBy :: (Eq obj, Eq mor) => Gog l obj mor -> 
              (Gog l obj mor -> obj -> obj -> [mor -> mor]) ->
              (Ded   obj mor -> obj -> [mor])               ->
              (Gog l obj mor -> obj -> obj -> [mor])        ->
              TestResult obj mor
gog_TESTBy g anti sc crisp =
   let top = gog_top g
       bot = gog_bottom g
   in  ffold [test (not (top src trg == bot src trg)) 
                   [src,trg] [top src trg,bot src trg] "Top==Bot:" 
              | src <- gog_objects g, trg <- gog_objects g] . 
       testUpDownAllBy g anti sc crisp .  
       ded_top_incl_TEST (gog_ded g)   .
       divAll_lres_TEST (gog_divAll g) .
       distrAll_TEST (gog_distrAll g)

gog_TEST :: (Eq obj, Eq mor) => Gog l obj mor -> TestResult obj mor
gog_TEST g = gog_TESTBy g antiMorph scalars crispRel
\end{code}
Linear Goguen categories are essential to get the equivalence of the three notions of crispness, %%@
s-crispness and l-crispness (cf.\ Theorem \ref{ThmEquivL-S-Crisp}). Thus, we provide predefined tests %%@
to check linearity within a Goguen category.
\begin{code}
testLinearBy :: Eq mor => Gog l obj mor -> (Ded obj mor -> obj -> [mor]) -> 
                          TestResult obj mor
testLinearBy g sc = 
    ffold [ let scG = sc (gog_ded g) o
                bG  = gog_bottom g o o
            in ffold [test (s1 == bG || gog_meet g s1 s2 /= bG || s2 == bG)
                           [o] [s1,s2] "Linearity:"
                      |s1 <- scG ,s2 <- scG]
            | o <- gog_objects g]

testLinear :: (Eq mor) => Gog l obj mor -> TestResult obj mor
testLinear g = testLinearBy g scalars
\end{code}
Obviously, the test is done by checking all scalars of all objects for the linearity property.

With all kinds of relational categories, functors (cf.\ Definition \ref{DefnFunc}) are essential to %%@
express interactions between them. Hence, we provide suitable tests to check whether a given functor %%@
is a functor between Goguen categories. RATH already provides the data structure \smalltt{Fun} shown %%@
in Section \ref{SubSecCat}. Unfortunately, suitable tests are only included up to the level of %%@
allegories so that we cannot rely on a preimplemented test for functors between Dedekind categories. %%@
Hence, we have to provide the missing test ourself.
\begin{code}
gog_fun_TEST :: Eq mor2 => Gog l obj1 mor1 -> Gog l obj2 mor2 -> 
                           Fun obj2 mor2 obj1 mor1 -> TestResult obj1 mor1
gog_fun_TEST g1 g2 f@(Fun fObj fMor) = 
  let os = gog_objects g1
  in ffold [ test (fMor (gog_join g1 m1 m2)==gog_join g2 (fMor m1) (fMor m2)) 
                  [o1,o2] [m1,m2] "Join:" .
             test (fMor (gog_lres g1 m1 m2)==gog_lres g2 (fMor m1) (fMor m2))
                  [o1,o2] [m1,m2] "LRes:"
             | o1<-os, o2<-os, m1<-gog_homset g1 o1 o2, m2<-gog_homset g1 o1 o2].
     ffold [ test (fMor (gog_up g1 m)==gog_up g2 (fMor m)) 
                  [o1,o2] [m] "Up:" .
             test (fMor (gog_down g1 m)==gog_down g2 (fMor m))
                  [o1,o2] [m] "Down:"
             | o1<-os, o2<-os, m<-gog_homset g1 o1 o2] .
     ffold [ test (fMor (gog_bottom g1 o1 o2)==gog_bottom g2 (fObj o1) (fObj o2)) 
                  [o1,o2] [gog_bottom g1 o1 o2] "Bottom:" .
             test (fMor (gog_top g1 o1 o2)==gog_top g2 (fObj o1) (fObj o2))
                  [o1,o2] [gog_top g1 o1 o2] "Top:"
             | o1<-os, o2<-os] .
     allrepr_TEST (gog_all g1) (gog_all g2) f
\end{code}
We check whether a given functor \smalltt{f} between two given Goguen categories \smalltt{g1} and %%@
\smalltt{g2} respects join and the left residual as well as the up/down operation and the top/bottom %%@
elements. The composition, conversion and meet operators are already tested by %%@
\smalltt{allrepf\_TEST}.

Finally, we again have to connect the class and module view, respectively, by providing suitable %%@
instances. A listing can be found in Appendix \ref{AppGogInstances}.

