\section{ A general extension for Goguen categories }
\label{SecExtGog}
In Section \ref{SecGoguenCat}, we gave the definition of a Goguen category introduced in %%@
\cite{WinterGog1}. We aim to provide a suitable computer-aided framework for this kind of category by %%@
extending the RATH module system. Fortunately, Goguen categories do not make any changes within the %%@
RATH modules necessary since the possibility to express crispness is not essential for the %%@
hierarchical definition of relational categories. As mentioned before, RATH uses two different views %%@
of relational categories --- the class view and the module view (brought about by record data %%@
structures). To guarantee consistency, we also differentiate here, and provide the modules %%@
\smalltt{GoguenClass}, \smalltt{Goguen} and \smalltt{GoguenInstances}.

First, we introduce a new type class for Goguen categories
\begin{code}
module GoguenClass where 

import RelAlgClasses (DedCat)

class DedCat gog obj mor => 
      GoguenCat gog loos obj mor | gog -> loos, gog -> obj, gog -> mor where
    up    :: gog -> mor -> mor
    down  :: gog -> mor -> mor
    derOp :: gog -> String -> loos -> mor -> mor -> mor 
\end{code}
such that the new operations $^{\ua}$ and $^{\da}$ (cf.\ Section \ref{SecGoguenCat}) are supported. %%@
Furthermore, derived operations play an important role within Goguen categories to reason over and %%@
represent fuzzy controllers. Hence, we provide the function \smalltt{derOp} which makes it possible %%@
to use as many user-defined operations as wanted. The certain operations are identified by a label %%@
and are parametrized by a given loos and the two arguments. Notice that the {\smalltt loos} data %%@
structure is still an abstract parameter within this module. We will have to provide a proper %%@
definition within our module for $\SL$-fuzzy relations later on.\\
Obviously, in conformance to the definition, our type class for Goguen categories is an extension of %%@
Dedekind categories. Thus, we have the new type class hierarchy shown in Figure \ref{FigRathClass}.
\begin{figure}[h]
\begin{center}
\begin{picture}(230,225)
\put(100,5  ){\framebox(100,15){\smalltt{Category}}}
\put(100,40 ){\framebox(100,15){\smalltt{Allegory}}}
\put(100,75 ){\framebox(100,15){\smalltt{DistribAllegory}}}
\put(100,110){\framebox(100,15){\smalltt{DivisionAllegory}}}
\put(100,145){\framebox(100,15){\smalltt{DedCat}}}
\put(100,210){\framebox(100,15){\smalltt{RelAlg}}}
\put(30,177 ){\framebox(100,15){\smalltt{GoguenCat}}}
\put(150,20 ){\line( 0,1){20}}
\put(150,55 ){\line( 0,1){20}}
\put(150,90 ){\line( 0,1){20}}
\put(150,125){\line( 0,1){20}}
\put(150,160){\line( 0,1){50}}
\put(100,160){\line(-1,1){17}}
\end{picture}
\end{center}
\caption{\label{FigRathClass} Extended type class hierarchy of RATH }
\end{figure}
