\chapter{ Goguen Instances and Export of the Functions }
\label{AppGogInstances}
In the following we want to provide instances for the class \smalltt{GoguenCat} provided in Section %%@
\ref{SecExtGog}. Furthermore, the according test routines shall be exported. Thus, we connect our %%@
\smalltt{Goguen} module and the RATH system.
\begin{code}
module GoguenInstances where 

import RelAlgClasses
import RelAlgInstances (dedDict)
import GoguenClass
import Goguen
import RelAlg
                              
instance Category         (Gog loos obj mor) obj mor where
    isObj   = gog_isObj
    isMor   = gog_isMor
    objects = gog_objects
    homset  = gog_homset
    source  = gog_source
    target  = gog_target
    idmor   = gog_idmor
    comp    = gog_comp
     
instance Allegory         (Gog loos obj mor) obj mor where
    converse = gog_converse
    meet     = gog_meet
    incl     = gog_incl
    
instance DistribAllegory  (Gog loos obj mor) obj mor where
    join   = gog_join
    bottom = gog_bottom
    
instance DivisionAllegory (Gog loos obj mor) obj mor where
    rres = gog_rres
    lres = gog_lres
    syq  = gog_syq
    
instance DedCat           (Gog loos obj mor) obj mor where
    top = gog_top
    
instance GoguenCat        (Gog loos obj mor) loos obj mor where 
    up    = gog_up
    down  = gog_down
    derOp = gog_derOp
\end{code}
Before we can export the tests, we need the reverse instance of the \smalltt{Goguen} class.
\begin{code}
gogDict :: GoguenCat g l obj mor => g -> Gog l obj mor
gogDict g = Gog { gog_up    = up g
                 ,gog_down  = down g
                 ,gog_derOp = derOp g }

\end{code}
Hence, the consistency tests and necessary functions are exported to be available within the type %%@
class view. Notice that we cannot export those functions that are parametrized by given functions to %%@
determine the scalars or crisp relations (e.g., \smalltt{antiMorphBy}). To do so, we would, for %%@
example, have to transform a function \smalltt{f} of type \\ 
\centerline{\smalltt{f::(DedCat d obj mor) => d -> obj -> [mor]}}
into a new function\\
\centerline{\smalltt{f':: Ded obj mor -> obj -> [mor]}}
which is, in general, not possible. Thus, we only export the tests using the standard functions for %%@
the determination of the scalars and crisp relations, respectively.
\begin{code}
t_antiMorph :: (Eq mor,GoguenCat g l obj mor)  => g -> obj -> obj -> [mor -> mor]
t_antiMorph g = antiMorph $ gogDict g

t_testAntiMorph    :: (Eq obj, Eq mor, GoguenCat g l obj mor) => 
                      g -> obj -> obj -> TestResult obj mor
t_testAntiMorph    g = testAntiMorph $ gogDict g

t_testAntiMorphAll :: (Eq obj,Eq mor,GoguenCat g l obj mor) => 
                      g -> TestResult obj mor
t_testAntiMorphAll g = testAntiMorphAll $ gogDict g

t_testUpDown :: (Eq obj, Eq mor, GoguenCat g l obj mor) => 
                g -> obj -> obj -> obj -> TestResult obj mor
t_testUpDown g = testUpDown $ gogDict g 

t_testUpDownAll :: (Eq obj, Eq mor, GoguenCat g l obj mor) => 
                   g -> TestResult obj mor
t_testUpDownAll g = testUpDownAll $ gogDict g 

goguen_TEST :: (Eq obj, Eq mor, GoguenCat g l obj mor) => g -> TestResult obj mor
goguen_TEST g = gog_TEST $ gogDict g

t_testLinear :: (Eq mor, GoguenCat g l obj mor) => g -> TestResult obj mor
t_testLinear g = testLinear $ gogDict g

goguen_fun_TEST :: (Eq mor2,GoguenCat g1 l obj1 mor1,GoguenCat g2 l obj2 mor2)=> 
                   g1 -> g2 -> Fun obj2 mor2 obj1 mor1 -> TestResult obj1 mor1
goguen_fun_TEST g1 g2 = gog_fun_TEST (gogDict g1) $ gogDict g2
\end{code}
